@@ -67,15 +67,16 @@ impl CoarseChannel {
         gpubox_number: usize,
         coarse_chan_width_hz: u32,
     ) -> Self {
-        let centre_chan_hz: u32 = (rec_chan_number as u32) * coarse_chan_width_hz;
+        let coarse_chan_centre_hz: u32 = (rec_chan_number as u32) * coarse_chan_width_hz;
+
         Self {
             corr_chan_number,
             rec_chan_number,
             gpubox_number,
             chan_width_hz: coarse_chan_width_hz,
-            chan_centre_hz: centre_chan_hz,
-            chan_start_hz: centre_chan_hz - (coarse_chan_width_hz / 2),
-            chan_end_hz: centre_chan_hz + (coarse_chan_width_hz / 2),
+            chan_centre_hz: coarse_chan_centre_hz,
+            chan_start_hz: coarse_chan_centre_hz - (coarse_chan_width_hz / 2),
+            chan_end_hz: coarse_chan_centre_hz + (coarse_chan_width_hz / 2),
         }
     }
     /// Takes the metafits long string of coarse channels, parses it and turns it into a vector
@@ -224,7 +225,7 @@ impl CoarseChannel {
                         _ => match voltage_time_map {
                             Some(v) => {
                                 if let Some((_, channel_map)) = v.iter().next() {
-                                    if channel_map.contains_key(&rec_chan_number) {
+                                    if channel_map.contains_key(rec_chan_number) {
                                         coarse_chans.push(CoarseChannel::new(
                                             correlator_chan_number,
                                             *rec_chan_number,
@@ -263,7 +264,7 @@ impl CoarseChannel {
                     match gpubox_time_map {
                         Some(g) => {
                             if let Some((_, channel_map)) = g.iter().next() {
-                                if channel_map.contains_key(&rec_chan_number) {
+                                if channel_map.contains_key(rec_chan_number) {
                                     coarse_chans.push(CoarseChannel::new(
                                         correlator_chan_number,
                                         *rec_chan_number,
@@ -276,7 +277,7 @@ impl CoarseChannel {
                         _ => match voltage_time_map {
                             Some(v) => {
                                 if let Some((_, channel_map)) = v.iter().next() {
-                                    if channel_map.contains_key(&rec_chan_number) {
+                                    if channel_map.contains_key(rec_chan_number) {
                                         coarse_chans.push(CoarseChannel::new(
                                             correlator_chan_number,
                                             *rec_chan_number,
@@ -335,6 +336,61 @@ impl CoarseChannel {
 
         coarse_chan_indices
     }
+
+    /// Calculate the centre frequency of each fine channel of this coarse channel.
+    ///
+    ///
+    /// # Arguments    
+    ///
+    /// * `mwa_version` - The version of the MWA is in use.
+    ///
+    /// * `coarse_channels` - Vector of populated Coarse Channels.
+    ///
+    /// * `fine_chan_width_hz` - Fine channel width in Hz.
+    ///
+    /// * `num_fine_chans_per_coarse` - Number of fine channels per coarse channel.
+    ///
+    /// # Returns
+    ///
+    /// * The centre frequency of the first fine channel of the coarse channel.
+    ///
+    pub fn get_fine_chan_centres_array_hz(
+        mwa_version: MWAVersion,
+        coarse_channels: &[CoarseChannel],
+        fine_chan_width_hz: u32,
+        num_fine_chans_per_coarse: usize,
+    ) -> Vec<f64> {
+        // Firstly calculate the offset
+        // For Legacy MWA, the offset is only needed if the fine channel width is 20 or 40kHz.
+        let offset_hz = match mwa_version {
+            MWAVersion::CorrLegacy
+            | MWAVersion::CorrOldLegacy
+            | MWAVersion::VCSLegacyRecombined => match num_fine_chans_per_coarse {
+                64 => 5_000.0,  // 20 kHz corr mode needs a 5 kHz offset applied
+                32 => 15_000.0, // 40 kHz corr mode needs a 15 kHz offset applied
+                _ => 0.0,       // other modes (10kHz) does not need any offset applied
+            },
+            MWAVersion::CorrMWAXv2 | MWAVersion::VCSMWAXv2 => 0.0,
+        };
+
+        // We need a factor based on whether the number of fine channels per coarse is even or odd
+        let odd_even_adjustment: f64 = match num_fine_chans_per_coarse % 2 == 0 {
+            true => 0.0,  // Even
+            false => 0.5, // Odd
+        };
+
+        // Return a vector of f64s which are the fine channel centre frequencies for all the fine channels in [coarse_channels]
+        coarse_channels
+            .iter()
+            .flat_map(|coarse_chan| {
+                (0..num_fine_chans_per_coarse).map(move |fine_chan_idx| {
+                    coarse_chan.chan_start_hz as f64
+                        + ((fine_chan_idx as f64 + odd_even_adjustment) * fine_chan_width_hz as f64)
+                        + offset_hz
+                })
+            })
+            .collect()
+    }
 }
 
 /// Implements fmt::Debug for
